@@ -1,38 +1,57 @@
 from __future__ import annotations
 
-from typing import TYPE_CHECKING, Any, cast, overload
+from typing import TYPE_CHECKING, Any, ClassVar, cast, overload
 
 import polars as pl
 
 from narwhals._polars.utils import (
+    BACKEND_VERSION,
+    SERIES_ACCEPTS_PD_INDEX,
+    SERIES_RESPECTS_DTYPE,
+    PolarsAnyNamespace,
+    PolarsCatNamespace,
+    PolarsDateTimeNamespace,
+    PolarsListNamespace,
+    PolarsStringNamespace,
+    PolarsStructNamespace,
     catch_polars_exception,
     extract_args_kwargs,
     extract_native,
     narwhals_to_native_dtype,
     native_to_narwhals_dtype,
 )
-from narwhals._utils import Implementation, requires, validate_backend_version
-from narwhals.dependencies import is_numpy_array_1d
+from narwhals._utils import Implementation, requires
+from narwhals.dependencies import is_numpy_array_1d, is_pandas_index
 
 if TYPE_CHECKING:
     from collections.abc import Iterable, Iterator, Mapping, Sequence
     from types import ModuleType
-    from typing import TypeVar
+    from typing import Literal, TypeVar
 
     import pandas as pd
     import pyarrow as pa
-    from typing_extensions import Self, TypeIs
+    from typing_extensions import Self, TypeAlias, TypeIs
 
     from narwhals._polars.dataframe import Method, PolarsDataFrame
-    from narwhals._polars.expr import PolarsExpr
     from narwhals._polars.namespace import PolarsNamespace
-    from narwhals._utils import Version, _FullContext
+    from narwhals._utils import Version, _LimitedContext
     from narwhals.dtypes import DType
     from narwhals.series import Series
-    from narwhals.typing import Into1DArray, IntoDType, MultiIndexSelector, _1DArray
+    from narwhals.typing import (
+        Into1DArray,
+        IntoDType,
+        ModeKeepStrategy,
+        MultiIndexSelector,
+        NonNestedLiteral,
+        NumericLiteral,
+        PythonLiteral,
+        _1DArray,
+    )
 
     T = TypeVar("T")
+    IncludeBreakpoint: TypeAlias = Literal[False, True]
 
+Incomplete: TypeAlias = Any
 
 # Series methods where PolarsSeries just defers to Polars.Series directly.
 INHERITED_METHODS = frozenset(
@@ -72,10 +91,14 @@ INHERITED_METHODS = frozenset(
         "drop_nulls",
         "exp",
         "fill_null",
+        "fill_nan",
         "filter",
         "gather_every",
         "head",
         "is_between",
+        "is_close",
+        "is_duplicated",
+        "is_empty",
         "is_finite",
         "is_first_distinct",
         "is_in",
@@ -115,14 +138,22 @@ INHERITED_METHODS = frozenset(
 
 
 class PolarsSeries:
-    def __init__(
-        self, series: pl.Series, *, backend_version: tuple[int, ...], version: Version
-    ) -> None:
-        self._native_series: pl.Series = series
-        self._backend_version = backend_version
-        self._implementation = Implementation.POLARS
+    _implementation: Implementation = Implementation.POLARS
+    _native_series: pl.Series
+    _version: Version
+
+    _HIST_EMPTY_SCHEMA: ClassVar[Mapping[IncludeBreakpoint, Sequence[str]]] = {
+        True: ["breakpoint", "count"],
+        False: ["count"],
+    }
+
+    def __init__(self, series: pl.Series, *, version: Version) -> None:
+        self._native_series = series
         self._version = version
-        validate_backend_version(self._implementation, self._backend_version)
+
+    @property
+    def _backend_version(self) -> tuple[int, ...]:
+        return self._implementation._backend_version()
 
     def __repr__(self) -> str:  # pragma: no cover
         return "PolarsSeries"
@@ -130,9 +161,7 @@ class PolarsSeries:
     def __narwhals_namespace__(self) -> PolarsNamespace:
         from narwhals._polars.namespace import PolarsNamespace
 
-        return PolarsNamespace(
-            backend_version=self._backend_version, version=self._version
-        )
+        return PolarsNamespace(version=self._version)
 
     def __narwhals_series__(self) -> Self:
         return self
@@ -145,27 +174,28 @@ class PolarsSeries:
         raise AssertionError(msg)
 
     def _with_version(self, version: Version) -> Self:
-        return self.__class__(
-            self.native, backend_version=self._backend_version, version=version
-        )
+        return self.__class__(self.native, version=version)
 
     @classmethod
     def from_iterable(
         cls,
         data: Iterable[Any],
         *,
-        context: _FullContext,
+        context: _LimitedContext,
         name: str = "",
         dtype: IntoDType | None = None,
     ) -> Self:
         version = context._version
-        backend_version = context._backend_version
-        dtype_pl = (
-            narwhals_to_native_dtype(dtype, version, backend_version) if dtype else None
-        )
-        # NOTE: `Iterable` is fine, annotation is overly narrow
-        # https://github.com/pola-rs/polars/blob/82d57a4ee41f87c11ca1b1af15488459727efdd7/py-polars/polars/series/series.py#L332-L333
-        native = pl.Series(name=name, values=cast("Sequence[Any]", data), dtype=dtype_pl)
+        dtype_pl = narwhals_to_native_dtype(dtype, version) if dtype else None
+        values: Incomplete = data
+        if SERIES_RESPECTS_DTYPE:
+            native = pl.Series(name, values, dtype=dtype_pl)
+        else:  # pragma: no cover
+            if (not SERIES_ACCEPTS_PD_INDEX) and is_pandas_index(values):
+                values = values.to_series()
+            native = pl.Series(name, values)
+            if dtype_pl:
+                native = native.cast(dtype_pl)
         return cls.from_native(native, context=context)
 
     @staticmethod
@@ -173,13 +203,11 @@ class PolarsSeries:
         return isinstance(obj, pl.Series)
 
     @classmethod
-    def from_native(cls, data: pl.Series, /, *, context: _FullContext) -> Self:
-        return cls(
-            data, backend_version=context._backend_version, version=context._version
-        )
+    def from_native(cls, data: pl.Series, /, *, context: _LimitedContext) -> Self:
+        return cls(data, version=context._version)
 
     @classmethod
-    def from_numpy(cls, data: Into1DArray, /, *, context: _FullContext) -> Self:
+    def from_numpy(cls, data: Into1DArray, /, *, context: _LimitedContext) -> Self:
         native = pl.Series(data if is_numpy_array_1d(data) else [data])
         return cls.from_native(native, context=context)
 
@@ -187,9 +215,7 @@ class PolarsSeries:
         return self._version.series(self, level="full")
 
     def _with_native(self, series: pl.Series) -> Self:
-        return self.__class__(
-            series, backend_version=self._backend_version, version=self._version
-        )
+        return self.__class__(series, version=self._version)
 
     @overload
     def _from_native_object(self, series: pl.Series) -> Self: ...
@@ -212,9 +238,6 @@ class PolarsSeries:
         # scalar
         return series
 
-    def _to_expr(self) -> PolarsExpr:
-        return self.__narwhals_namespace__()._expr._from_series(self)
-
     def __getattr__(self, attr: str) -> Any:
         if attr not in INHERITED_METHODS:
             msg = f"{self.__class__.__name__} has not attribute '{attr}'."
@@ -229,15 +252,24 @@ class PolarsSeries:
     def __len__(self) -> int:
         return len(self.native)
 
+    def __rfloordiv__(self, other: Any) -> PolarsSeries:
+        if self._backend_version < (1, 10, 0):
+            name = self.name
+            ns = self.__narwhals_namespace__()
+            return (
+                self.to_frame()
+                .select((ns.col(name).__rfloordiv__(other)).alias(name))
+                .get_column(name)
+            )
+        return self._with_native(self.native.__rfloordiv__(extract_native(other)))
+
     @property
     def name(self) -> str:
         return self.native.name
 
     @property
     def dtype(self) -> DType:
-        return native_to_narwhals_dtype(
-            self.native.dtype, self._version, self._backend_version
-        )
+        return native_to_narwhals_dtype(self.native.dtype, self._version)
 
     @property
     def native(self) -> pl.Series:
@@ -252,7 +284,7 @@ class PolarsSeries:
         return self._from_native_object(self.native.__getitem__(item))
 
     def cast(self, dtype: IntoDType) -> Self:
-        dtype_pl = narwhals_to_native_dtype(dtype, self._version, self._backend_version)
+        dtype_pl = narwhals_to_native_dtype(dtype, self._version)
         return self._with_native(self.native.cast(dtype_pl))
 
     @requires.backend_version((1,))
@@ -265,7 +297,7 @@ class PolarsSeries:
     ) -> Self:
         ser = self.native
         dtype = (
-            narwhals_to_native_dtype(return_dtype, self._version, self._backend_version)
+            narwhals_to_native_dtype(return_dtype, self._version)
             if return_dtype
             else None
         )
@@ -285,18 +317,18 @@ class PolarsSeries:
     def __ne__(self, other: object) -> Self:  # type: ignore[override]
         return self._with_native(self.native.__ne__(extract_native(other)))
 
-    # NOTE: `pyright` is being reasonable here
-    def __ge__(self, other: Any) -> Self:
-        return self._with_native(self.native.__ge__(extract_native(other)))  # pyright: ignore[reportArgumentType]
+    # NOTE: These need to be anything that can't match `PolarsExpr`, due to overload order
+    def __ge__(self, other: Self) -> Self:
+        return self._with_native(self.native.__ge__(extract_native(other)))
 
-    def __gt__(self, other: Any) -> Self:
-        return self._with_native(self.native.__gt__(extract_native(other)))  # pyright: ignore[reportArgumentType]
+    def __gt__(self, other: Self) -> Self:
+        return self._with_native(self.native.__gt__(extract_native(other)))
 
-    def __le__(self, other: Any) -> Self:
-        return self._with_native(self.native.__le__(extract_native(other)))  # pyright: ignore[reportArgumentType]
+    def __le__(self, other: Self) -> Self:
+        return self._with_native(self.native.__le__(extract_native(other)))
 
-    def __lt__(self, other: Any) -> Self:
-        return self._with_native(self.native.__lt__(extract_native(other)))  # pyright: ignore[reportArgumentType]
+    def __lt__(self, other: Self) -> Self:
+        return self._with_native(self.native.__lt__(extract_native(other)))
 
     def __rpow__(self, other: PolarsSeries | Any) -> Self:
         result = self.native.__rpow__(extract_native(other))
@@ -309,7 +341,7 @@ class PolarsSeries:
         try:
             native_is_nan = self.native.is_nan()
         except Exception as e:  # noqa: BLE001
-            raise catch_polars_exception(e, self._backend_version) from None
+            raise catch_polars_exception(e) from None
         if self._backend_version < (1, 18):  # pragma: no cover
             select = pl.when(self.native.is_not_null()).then(native_is_nan)
             return self._with_native(pl.select(select)[self.name])
@@ -468,83 +500,120 @@ class PolarsSeries:
         return PolarsDataFrame.from_native(result, context=self)
 
     def cum_count(self, *, reverse: bool) -> Self:
-        if self._backend_version < (0, 20, 4):
-            not_null_series = ~self.native.is_null()
-            result = not_null_series.cum_sum(reverse=reverse)
-        else:
-            result = self.native.cum_count(reverse=reverse)
-
-        return self._with_native(result)
+        return self._with_native(self.native.cum_count(reverse=reverse))
 
     def __contains__(self, other: Any) -> bool:
         try:
             return self.native.__contains__(other)
         except Exception as e:  # noqa: BLE001
-            raise catch_polars_exception(e, self._backend_version) from None
+            raise catch_polars_exception(e) from None
 
-    def hist(  # noqa: C901, PLR0912
+    def is_close(
         self,
-        bins: list[float | int] | None,
+        other: Self | NumericLiteral,
         *,
-        bin_count: int | None,
-        include_breakpoint: bool,
+        abs_tol: float,
+        rel_tol: float,
+        nans_equal: bool,
+    ) -> PolarsSeries:
+        if self._backend_version < (1, 32, 0):
+            name = self.name
+            ns = self.__narwhals_namespace__()
+            other_expr = (
+                ns.lit(other.native, None) if isinstance(other, PolarsSeries) else other
+            )
+            expr = ns.col(name).is_close(
+                other_expr, abs_tol=abs_tol, rel_tol=rel_tol, nans_equal=nans_equal
+            )
+            return self.to_frame().select(expr).get_column(name)
+        other_series = other.native if isinstance(other, PolarsSeries) else other
+        result = self.native.is_close(
+            other_series, abs_tol=abs_tol, rel_tol=rel_tol, nans_equal=nans_equal
+        )
+        return self._with_native(result)
+
+    def mode(self, *, keep: ModeKeepStrategy) -> Self:
+        result = self.native.mode()
+        return self._with_native(result.head(1) if keep == "any" else result)
+
+    def hist_from_bins(
+        self, bins: list[float], *, include_breakpoint: bool
     ) -> PolarsDataFrame:
+        if len(bins) <= 1:
+            native = pl.DataFrame(schema=self._HIST_EMPTY_SCHEMA[include_breakpoint])
+        elif self.native.is_empty():
+            if include_breakpoint:
+                native = (
+                    pl.Series(bins[1:])
+                    .to_frame("breakpoint")
+                    .with_columns(count=pl.lit(0, pl.Int64))
+                )
+            else:
+                native = pl.select(count=pl.zeros(len(bins) - 1, pl.Int64))
+        else:
+            return self._hist_from_data(
+                bins=bins, bin_count=None, include_breakpoint=include_breakpoint
+            )
+        return self.__narwhals_namespace__()._dataframe.from_native(native, context=self)
+
+    def hist_from_bin_count(
+        self, bin_count: int, *, include_breakpoint: bool
+    ) -> PolarsDataFrame:
+        if bin_count == 0:
+            native = pl.DataFrame(schema=self._HIST_EMPTY_SCHEMA[include_breakpoint])
+        elif self.native.is_empty():
+            if include_breakpoint:
+                native = pl.select(
+                    breakpoint=pl.int_range(1, bin_count + 1) / bin_count,
+                    count=pl.lit(0, pl.Int64),
+                )
+            else:
+                native = pl.select(count=pl.zeros(bin_count, pl.Int64))
+        else:
+            count: int | None
+            if BACKEND_VERSION < (1, 15):  # pragma: no cover
+                count = None
+                bins = self._bins_from_bin_count(bin_count=bin_count)
+            else:
+                count = bin_count
+                bins = None
+            return self._hist_from_data(
+                bins=bins,  # type: ignore[arg-type]
+                bin_count=count,
+                include_breakpoint=include_breakpoint,
+            )
+        return self.__narwhals_namespace__()._dataframe.from_native(native, context=self)
+
+    def _bins_from_bin_count(self, bin_count: int) -> pl.Series:  # pragma: no cover
+        """Prepare bins based on backend version compatibility.
+
+        polars <1.15 does not adjust the bins when they have equivalent min/max
+        polars <1.5 with bin_count=...
+        returns bins that range from -inf to +inf and has bin_count + 1 bins.
+          for compat: convert `bin_count=` call to `bins=`
+        """
+        lower = cast("float", self.native.min())
+        upper = cast("float", self.native.max())
+
+        if lower == upper:
+            lower -= 0.5
+            upper += 0.5
+
+        width = (upper - lower) / bin_count
+        return pl.int_range(0, bin_count + 1, eager=True) * width + lower
+
+    def _hist_from_data(
+        self, bins: list[float] | None, bin_count: int | None, *, include_breakpoint: bool
+    ) -> PolarsDataFrame:
+        """Calculate histogram from non-empty data and post-process the results based on the backend version."""
         from narwhals._polars.dataframe import PolarsDataFrame
 
-        if (bins is not None and len(bins) <= 1) or (bin_count == 0):  # pragma: no cover
-            data: list[pl.Series] = []
-            if include_breakpoint:
-                data.append(pl.Series("breakpoint", [], dtype=pl.Float64))
-            data.append(pl.Series("count", [], dtype=pl.UInt32))
-            return PolarsDataFrame.from_native(pl.DataFrame(data), context=self)
-
-        if self.native.count() < 1:
-            data_dict: dict[str, Sequence[Any] | pl.Series]
-            if bins is not None:
-                data_dict = {
-                    "breakpoint": bins[1:],
-                    "count": pl.zeros(n=len(bins) - 1, dtype=pl.Int64, eager=True),
-                }
-            elif (bin_count is not None) and bin_count == 1:
-                data_dict = {"breakpoint": [1.0], "count": [0]}
-            elif (bin_count is not None) and bin_count > 1:
-                data_dict = {
-                    "breakpoint": pl.int_range(1, bin_count + 1, eager=True) / bin_count,
-                    "count": pl.zeros(n=bin_count, dtype=pl.Int64, eager=True),
-                }
-            else:  # pragma: no cover
-                msg = (
-                    "congratulations, you entered unreachable code - please report a bug"
-                )
-                raise AssertionError(msg)
-            if not include_breakpoint:
-                del data_dict["breakpoint"]
-            return PolarsDataFrame.from_native(pl.DataFrame(data_dict), context=self)
-
-        # polars <1.15 does not adjust the bins when they have equivalent min/max
-        # polars <1.5 with bin_count=...
-        # returns bins that range from -inf to +inf and has bin_count + 1 bins.
-        #   for compat: convert `bin_count=` call to `bins=`
-        if (self._backend_version < (1, 15)) and (
-            bin_count is not None
-        ):  # pragma: no cover
-            lower = cast("float", self.native.min())
-            upper = cast("float", self.native.max())
-            if lower == upper:
-                width = 1 / bin_count
-                lower -= 0.5
-                upper += 0.5
-            else:
-                width = (upper - lower) / bin_count
-
-            bins = (pl.int_range(0, bin_count + 1, eager=True) * width + lower).to_list()
-            bin_count = None
+        series = self.native
 
         # Polars inconsistently handles NaN values when computing histograms
         #   against predefined bins: https://github.com/pola-rs/polars/issues/21082
-        series = self.native
-        if self._backend_version < (1, 15) or bins is not None:
-            series = series.set(series.is_nan(), None)
+        if BACKEND_VERSION < (1, 15) or bins is not None:
+            series = series.fill_nan(None)
 
         df = series.hist(
             bins,
@@ -553,26 +622,43 @@ class PolarsSeries:
             include_breakpoint=include_breakpoint,
         )
 
-        if not include_breakpoint:
-            df.columns = ["count"]
+        # Apply post-processing corrections
 
-        if self._backend_version < (1, 0) and include_breakpoint:
+        # Handle column naming
+        if not include_breakpoint:
+            col_name = df.columns[0]
+            df = df.select(pl.col(col_name).alias("count"))
+        elif BACKEND_VERSION < (1, 0):  # pragma: no cover
             df = df.rename({"break_point": "breakpoint"})
 
-        #  polars<1.15 implicitly adds -inf and inf to either end of bins
-        if self._backend_version < (1, 15) and bins is not None:  # pragma: no cover
-            r = pl.int_range(0, len(df))
-            df = df.filter((r > 0) & (r < len(df) - 1))
-
-        # polars<1.27 makes the lowest bin a left/right closed interval.
-        if self._backend_version < (1, 27) and bins is not None:
-            df[0, "count"] += (series == bins[0]).sum()
+        if bins is not None:  # pragma: no cover
+            # polars<1.6 implicitly adds -inf and inf to either end of bins
+            if BACKEND_VERSION < (1, 6):
+                r = pl.int_range(0, len(df))
+                df = df.filter((r > 0) & (r < len(df) - 1))
+            # polars<1.27 makes the lowest bin a left/right closed interval
+            if BACKEND_VERSION < (1, 27):
+                df = (
+                    df.slice(0, 1)
+                    .with_columns(pl.col("count") + ((pl.lit(series) == bins[0]).sum()))
+                    .vstack(df.slice(1))
+                )
 
         return PolarsDataFrame.from_native(df, context=self)
 
     def to_polars(self) -> pl.Series:
         return self.native
 
+    def first(self) -> PythonLiteral:
+        if self._backend_version < (1, 10):  # pragma: no cover
+            return self.native.item(0) if len(self) else None
+        return self.native.first()  # type: ignore[return-value]
+
+    def last(self) -> PythonLiteral:
+        if self._backend_version < (1, 10):  # pragma: no cover
+            return self.native.item(-1) if len(self) else None
+        return self.native.last()  # type: ignore[return-value]
+
     @property
     def dt(self) -> PolarsSeriesDateTimeNamespace:
         return PolarsSeriesDateTimeNamespace(self)
@@ -600,7 +686,6 @@ class PolarsSeries:
     __pow__: Method[Self]
     __radd__: Method[Self]
     __rand__: Method[Self]
-    __rfloordiv__: Method[Self]
     __rmod__: Method[Self]
     __rmul__: Method[Self]
     __ror__: Method[Self]
@@ -624,10 +709,13 @@ class PolarsSeries:
     drop_nulls: Method[Self]
     exp: Method[Self]
     fill_null: Method[Self]
+    fill_nan: Method[Self]
     filter: Method[Self]
     gather_every: Method[Self]
     head: Method[Self]
     is_between: Method[Self]
+    is_duplicated: Method[Self]
+    is_empty: Method[bool]
     is_finite: Method[Self]
     is_first_distinct: Method[Self]
     is_in: Method[Self]
@@ -642,7 +730,6 @@ class PolarsSeries:
     max: Method[Any]
     mean: Method[float]
     min: Method[Any]
-    mode: Method[Self]
     n_unique: Method[int]
     null_count: Method[int]
     quantile: Method[float]
@@ -668,92 +755,67 @@ class PolarsSeries:
         return PolarsSeriesListNamespace(self)
 
 
-class PolarsSeriesDateTimeNamespace:
-    def __init__(self, series: PolarsSeries) -> None:
-        self._compliant_series = series
-
-    def __getattr__(self, attr: str) -> Any:
-        def func(*args: Any, **kwargs: Any) -> Any:
-            pos, kwds = extract_args_kwargs(args, kwargs)
-            return self._compliant_series._with_native(
-                getattr(self._compliant_series.native.dt, attr)(*pos, **kwds)
-            )
-
-        return func
-
-
-class PolarsSeriesStringNamespace:
-    def __init__(self, series: PolarsSeries) -> None:
-        self._compliant_series = series
-
-    def zfill(self, width: int) -> PolarsSeries:
-        series = self._compliant_series
-        name = series.name
-        ns = series.__narwhals_namespace__()
-        return series.to_frame().select(ns.col(name).str.zfill(width)).get_column(name)
-
-    def __getattr__(self, attr: str) -> Any:
-        def func(*args: Any, **kwargs: Any) -> Any:
-            pos, kwds = extract_args_kwargs(args, kwargs)
-            return self._compliant_series._with_native(
-                getattr(self._compliant_series.native.str, attr)(*pos, **kwds)
-            )
-
-        return func
-
-
-class PolarsSeriesCatNamespace:
-    def __init__(self, series: PolarsSeries) -> None:
-        self._compliant_series = series
-
-    def __getattr__(self, attr: str) -> Any:
-        def func(*args: Any, **kwargs: Any) -> Any:
-            pos, kwds = extract_args_kwargs(args, kwargs)
-            return self._compliant_series._with_native(
-                getattr(self._compliant_series.native.cat, attr)(*pos, **kwds)
-            )
-
-        return func
-
-
-class PolarsSeriesListNamespace:
+class PolarsSeriesNamespace(PolarsAnyNamespace[PolarsSeries, pl.Series]):
     def __init__(self, series: PolarsSeries) -> None:
         self._series = series
 
+    @property
+    def compliant(self) -> PolarsSeries:
+        return self._series
+
+    @property
+    def native(self) -> pl.Series:
+        return self._series.native
+
+    @property
+    def name(self) -> str:
+        return self.compliant.name
+
+    def __narwhals_namespace__(self) -> PolarsNamespace:
+        return self.compliant.__narwhals_namespace__()
+
+    def to_frame(self) -> PolarsDataFrame:
+        return self.compliant.to_frame()
+
+
+class PolarsSeriesDateTimeNamespace(
+    PolarsSeriesNamespace, PolarsDateTimeNamespace[PolarsSeries, pl.Series]
+): ...
+
+
+class PolarsSeriesStringNamespace(
+    PolarsSeriesNamespace, PolarsStringNamespace[PolarsSeries, pl.Series]
+):
+    def to_titlecase(self) -> PolarsSeries:
+        name = self.name
+        ns = self.__narwhals_namespace__()
+        return self.to_frame().select(ns.col(name).str.to_titlecase()).get_column(name)
+
+    def zfill(self, width: int) -> PolarsSeries:
+        name = self.name
+        ns = self.__narwhals_namespace__()
+        return self.to_frame().select(ns.col(name).str.zfill(width)).get_column(name)
+
+
+class PolarsSeriesCatNamespace(
+    PolarsSeriesNamespace, PolarsCatNamespace[PolarsSeries, pl.Series]
+): ...
+
+
+class PolarsSeriesListNamespace(
+    PolarsSeriesNamespace, PolarsListNamespace[PolarsSeries, pl.Series]
+):
     def len(self) -> PolarsSeries:
-        native_series = self._series.native
-        native_result = native_series.list.len()
+        name = self.name
+        ns = self.__narwhals_namespace__()
+        return self.to_frame().select(ns.col(name).list.len()).get_column(name)
 
-        if self._series._backend_version < (1, 16):  # pragma: no cover
-            native_result = pl.select(
-                pl.when(~native_series.is_null()).then(native_result).otherwise(None)
-            )[native_series.name].cast(pl.UInt32())
-
-        elif self._series._backend_version < (1, 17):  # pragma: no cover
-            native_result = native_series.cast(pl.UInt32())
-
-        return self._series._with_native(native_result)
-
-    # TODO(FBruzzesi): Remove `pragma: no cover` once other namespace methods are added
-    def __getattr__(self, attr: str) -> Any:  # pragma: no cover
-        def func(*args: Any, **kwargs: Any) -> Any:
-            pos, kwds = extract_args_kwargs(args, kwargs)
-            return self._series._with_native(
-                getattr(self._series.native.list, attr)(*pos, **kwds)
-            )
-
-        return func
+    def contains(self, item: NonNestedLiteral) -> PolarsSeries:
+        name = self.name
+        ns = self.__narwhals_namespace__()
+        return self.to_frame().select(ns.col(name).list.contains(item)).get_column(name)
 
 
-class PolarsSeriesStructNamespace:
-    def __init__(self, series: PolarsSeries) -> None:
-        self._compliant_series = series
-
-    def __getattr__(self, attr: str) -> Any:
-        def func(*args: Any, **kwargs: Any) -> Any:
-            pos, kwds = extract_args_kwargs(args, kwargs)
-            return self._compliant_series._with_native(
-                getattr(self._compliant_series.native.struct, attr)(*pos, **kwds)
-            )
-
-        return func
+class PolarsSeriesStructNamespace(
+    PolarsSeriesNamespace, PolarsStructNamespace[PolarsSeries, pl.Series]
+): ...
